@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -21,6 +20,7 @@ from calibre.gui2.tweak_book.editor.text import TextEdit
 from calibre.utils.icu import sort_key
 from calibre.web.feeds.recipes.collection import get_builtin_recipe_collection, get_builtin_recipe_by_id
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import iteritems, unicode_type, range, as_unicode
 
 
 def is_basic_recipe(src):
@@ -69,7 +69,7 @@ class CustomRecipeModel(QAbstractListModel):  # {{{
 
     def replace_many_by_title(self, scriptmap):
         script_urn_map = {}
-        for title, script in scriptmap.iteritems():
+        for title, script in iteritems(scriptmap):
             urn = None
             for x in self.recipe_model.custom_recipe_collection:
                 if x.get('title', False) == title:
@@ -118,14 +118,14 @@ def py3_repr(x):
     ans = repr(x)
     if isinstance(x, bytes) and not ans.startswith('b'):
         ans = 'b' + ans
-    if isinstance(x, unicode) and ans.startswith('u'):
+    if isinstance(x, unicode_type) and ans.startswith('u'):
         ans = ans[1:]
     return ans
 
 
 def options_to_recipe_source(title, oldest_article, max_articles_per_feed, feeds):
     classname = 'BasicUserRecipe%d' % int(time.time())
-    title = unicode(title).strip() or classname
+    title = unicode_type(title).strip() or classname
     indent = ' ' * 8
     if feeds:
         if len(feeds[0]) == 1:
@@ -377,32 +377,31 @@ class BasicRecipe(QWidget):  # {{{
             return False
         return True
 
-    @dynamic_property
+    @property
     def recipe_source(self):
 
-        def fget(self):
-            title = self.title.text().strip()
-            feeds = [self.feeds.item(i).data(Qt.UserRole) for i in xrange(self.feeds.count())]
-            return options_to_recipe_source(title, self.oldest_article.value(), self.max_articles.value(), feeds)
+        title = self.title.text().strip()
+        feeds = [self.feeds.item(i).data(Qt.UserRole) for i in range(self.feeds.count())]
+        return options_to_recipe_source(title, self.oldest_article.value(), self.max_articles.value(), feeds)
 
-        def fset(self, src):
-            self.feeds.clear()
-            self.feed_title.clear()
-            self.feed_url.clear()
-            if src is None:
-                self.title.setText(_('My news source'))
-                self.oldest_article.setValue(7)
-                self.max_articles.setValue(100)
-            else:
-                recipe = compile_recipe(src)
-                self.title.setText(recipe.title)
-                self.oldest_article.setValue(recipe.oldest_article)
-                self.max_articles.setValue(recipe.max_articles_per_feed)
-                for x in (recipe.feeds or ()):
-                    title, url = ('', x) if len(x) == 1 else x
-                    QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.UserRole, (title, url))
+    @recipe_source.setter
+    def recipe_source(self, src):
+        self.feeds.clear()
+        self.feed_title.clear()
+        self.feed_url.clear()
+        if src is None:
+            self.title.setText(_('My news source'))
+            self.oldest_article.setValue(7)
+            self.max_articles.setValue(100)
+        else:
+            recipe = compile_recipe(src)
+            self.title.setText(recipe.title)
+            self.oldest_article.setValue(recipe.oldest_article)
+            self.max_articles.setValue(recipe.max_articles_per_feed)
+            for x in (recipe.feeds or ()):
+                title, url = ('', x) if len(x) == 1 else x
+                QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.UserRole, (title, url))
 
-        return property(fget=fget, fset=fset)
 # }}}
 
 
@@ -430,16 +429,13 @@ class AdvancedRecipe(QWidget):  # {{{
             return False
         return True
 
-    @dynamic_property
+    @property
     def recipe_source(self):
+        return self.editor.toPlainText()
 
-        def fget(self):
-            return self.editor.toPlainText()
-
-        def fset(self, src):
-            self.editor.load_text(src, syntax='python', doc_name='<recipe>')
-
-        return property(fget=fget, fset=fset)
+    @recipe_source.setter
+    def recipe_source(self, src):
+        self.editor.load_text(src, syntax='python', doc_name='<recipe>')
 
     def sizeHint(self):
         return QSize(800, 500)
@@ -593,11 +589,12 @@ class CustomRecipes(Dialog):
         if not items:
             return
         item = items[-1]
-        id_ = unicode(item.data(Qt.UserRole) or '')
-        title = unicode(item.data(Qt.DisplayRole) or '').rpartition(' [')[0]
+        id_ = unicode_type(item.data(Qt.UserRole) or '')
+        title = unicode_type(item.data(Qt.DisplayRole) or '').rpartition(' [')[0]
         src = get_builtin_recipe_by_id(id_, download_recipe=True)
         if src is None:
             raise Exception('Something weird happened')
+        src = as_unicode(src)
 
         self.edit_recipe(None, src)
 
@@ -649,7 +646,7 @@ class CustomRecipes(Dialog):
         if replace_recipes:
             self.recipe_list.replace_many_by_title(replace_recipes)
         if failed_recipes:
-            det_msg = '\n'.join('%s\n%s\n' % (title, tb) for title, tb in failed_recipes.iteritems())
+            det_msg = '\n'.join('%s\n%s\n' % (title, tb) for title, tb in iteritems(failed_recipes))
             error_dialog(self, _('Failed to create recipes'), _(
                 'Failed to create some recipes, click "Show details" for details'), show=True,
                          det_msg=det_msg)
